/** Le istanze di questa classe rappresentano frazioni ridotte ai minimi termini.
 * 
 * @author Paolo Boldi
 *
 */
public class Frazione {
	// ATTRIBUTI
	/** Questo &egrave; il numeratore eventualmente negativo della frazione dopo la semplificazione. */
	private int num;
	/** Questo &egrave; il denominatore positivo della frazione dopo la semplificazione. */
	private int den;

	// COSTRUTTORI
	/** Costruisce una frazione con i parametri dati, semplificandola.
	 * 
	 * @param x il numeratore (prima della eventuale semplificazione).
	 * @param y il denominatore (prima della eventuale semplificazione).
	 * @throws DenominatoreNulloException se y &egrave; 0.
	 */
	public Frazione(int x, int y) throws DenominatoreNulloException {
		if (y==0) throw new DenominatoreNulloException();
		int r = MyMath.gcd(Math.abs(x), Math.abs(y));
		num = Math.abs(x) / r;
		den = Math.abs(y) / r;
		if ((x < 0 && y > 0) || (x > 0 && y < 0))   /// if (x*y < 0) 
			num = -num;
	}

	/** Costruisce la frazione x/1.
	 * 
	 * @param x il numeratore della frazione.
	 * @throws DenominatoreNulloException  
	 */
	public Frazione(int x) throws DenominatoreNulloException {
		this(x, 1);
	}

	/** Restituisce il numeratore.
	 * 
	 * @return il numeratore.
	 */
	public int getNumeratore() {
		return this.num;
	}
	
	/** Restituisce il denominatore.
	 * 
	 * @return il denominatore.
	 */
	public int getDenominatore() {
		return this.den;
	}

	/** Restituisce il prodotto fra la frazione su cui &egrave; invocato e quella passata come argomento.
	 * 
	 * @param f uno delle due frazioni coinvolte (l'altra &egrave; il target dell'invocazione).
	 * @return il prodotto.
	 */
	public Frazione per(Frazione f) {
		int x, y;
		x = f.num * this.num;
		y = f.den * this.den;
		Frazione ris = null;
		try {
			ris = new Frazione(x, y);
		} catch (DenominatoreNulloException e) {
			// CAN'T HAPPEN
		}
		return ris;
	}

	/** Restituisce la somma fra la frazione su cui &egrave; invocato e quella passata come argomento.
	 * 
	 * @param f uno delle due frazioni coinvolte (l'altra &egrave; il target dell'invocazione).
	 * @return la somma.
	 */
	public Frazione piu(Frazione f) {
		int x, y;
		x = f.num * this.den;
		y = f.den * this.num;
		Frazione ris = null;
		try {
			ris = new Frazione(x + y, f.den * this.den);
		} catch (DenominatoreNulloException e) {
			// CAN'T HAPPEN
		}
		return ris;
	}

	/** Restituisce una frazione opposta a questa.
	 *  
	 * @return la frazione opposta.
	 */
	private Frazione opposta() {
		try {
			return new Frazione(-num, den);
		} catch (DenominatoreNulloException e) {
			// CAN'T HAPPEN
			return null;
		}
	}

	/** Restituisce una frazione reciproca di questa.
	 *  
	 * @return la frazione reciproca.
	 */
	private Frazione reciproca() throws DenominatoreNulloException {
		return new Frazione(den, num);
	}
	
	/** Restituisce la differenza fra la frazione su cui &egrave; invocato e quella passata come argomento.
	 * 
	 * @param f la seconda delle due frazioni coinvolte (l'altra &egrave; il target dell'invocazione).
	 * @return la differenza.
	 */
	public Frazione meno(Frazione f) {
		return piu(f.opposta());
	}

	/** Restituisce il rapporto fra la frazione su cui &egrave; invocato e quella passata come argomento.
	 * 
	 * @param f la seconda delle due frazioni coinvolte (l'altra &egrave; il target dell'invocazione).
	 * @return il rapporto.
	 * @throws DenominatoreNulloException se f &egrave; la frazione nulla
	 */
	public Frazione diviso(Frazione f) throws DenominatoreNulloException {
		return per(f.reciproca());
	}

	/** Determina se questa frazione &egrave; negativa.
	 * 
	 * @return true sse questa frazione &egrave; negativa.
	 */
	public boolean isNegativa() {
		return this.num < 0;
	}

	/** Determina se questa frazione &egrave; nulla.
	 * 
	 * @return true sse questa frazione &egrave; nulla.
	 */
	public boolean isZero() {
		return this.num == 0;
	}

	/** Determina se questa frazione &egrave; minore di una passata come argomento.
	 * 
	 * @param f la frazione con cui confrontare questa.
	 * @return true sse questa frazione &egrave; minore di f.
	 */
	public boolean isMinore(Frazione f) {
		return this.meno(f).isNegativa();
	}


	@Override
	public String toString() {
		if (this.den > 1)
			return this.num + "/" + this.den;
		else
			return String.valueOf(this.num);
	}
	
	@Override
	public int hashCode() {
		return num + den;
	}
	
	@Override
	public boolean equals(Object x) {
		if (x instanceof Frazione) 
			return this.meno((Frazione)x).isZero();
		else
			return false;
		
}






}
